<?php

namespace Drupal\xnttexif\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\external_entities\ResponseDecoder\ResponseDecoderFactoryInterface;
use Drupal\xnttfiles\Plugin\ExternalEntities\StorageClient\Files;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * External entities storage client for image files.
 *
 * @ExternalEntityStorageClient(
 *   id = "xnttexif",
 *   label = @Translation("Image Files with exif data"),
 *   description = @Translation("Retrieves exif data from image files.")
 * )
 */
class ExifFiles extends Files {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationInterface $string_translation,
    ResponseDecoderFactoryInterface $response_decoder_factory,
    ConfigFactory $config_factory,
    MessengerInterface $messenger,
    LoggerChannelInterface $logger,
    CacheBackendInterface $cache
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $string_translation,
      $response_decoder_factory,
      $config_factory,
      $messenger,
      $logger,
      $cache
    );
    // Defaults.
    $this->fileType = 'image';
    $this->fileTypePlural = 'images';
    $this->fileTypeCap = 'Image';
    $this->fileTypeCapPlural = 'Images';
    // Default to JPEG but it could be PNG, GIF, TIFF, etc.
    $this->fileExtension = '.jpg';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('external_entities.response_decoder_factory'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('logger.channel.xnttexif'),
      $container->get('cache.default')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function parseFile(string $file_path) :array {
    $data = parent::parseFile($file_path);
    $id = array_keys($data)[0];
    // Add Exif infos.
    if (function_exists('exif_read_data')) {
      $exif_data = exif_read_data($file_path, '', TRUE, TRUE) ?: [];
      // Flatten array.
      $flatten_array = function ($array, $prefix = '') use (&$flatten_array) {
        $flattened = [];
        foreach ($array as $key => $value) {
          if (is_array($value)) {
            $flattened += $flatten_array($value, $prefix . $key . '.');
          }
          else {
            $flattened[$prefix . $key] = $value;
          }
        }
        return $flattened;
      };
      $data[$id] += $flatten_array($exif_data);
    }
    else {
      $this->messenger->addWarning(
        $this->t(
          'Unable to load exif data: PHP exif extension is not installed or enabled.'
        )
      );
      $this->logger->warning(
        'Unable to load exif data from file "'
        . $file_path
        . '": PHP exif extension is not installed or enabled.'
      );
    }
    return $data;
  }
}
